<?php
/**
 * @file
 * A hiearchy style plugin.
 */

/**
 * This style can show groups of content in hierarchies.
 */
class EntityCollectionStyleHierarchyLimited extends EntityCollectionStyleHierarchy {
  private $break_default = 10;

  /**
   * Build the hierarchy.
   * @see EntityCollectionStyle::build().
   */
  public function build(EntityCollection $entity, EntityCollectionTreeNode &$tree, $langcode = NULL, $settings = array()) {
    $this->settings = array_merge($this->settings, $settings);

    // FIXME/TODO: Should be placed someplace more generic.
    $entity->settings['style_settings'] = $this->settings;

    if ( $this->useBreakpoint() ) {
      $offset = 0;
      $tree = $tree->splice($offset, $this->getBreakpoint($entity));
    }

    // This will be used to remember if a part of the tree has already been rendered.
    if (!isset($tree->processed_offset)) {
      $tree->processed_offset = 0;
    }

    $limitation = $this->getLimits();
    if (!empty($limitation)) {

      if ($limitation['offset']) {
        $tree->processed_offset += $limitation['offset'];
      }

      $new_tree = clone $tree;
      $new_tree->splice($tree->processed_offset, $limitation['length']);
      $tree->processed_offset += $limitation['length'];
    }

    $tree_to_pass = isset($new_tree) ? $new_tree : $tree;
    return parent::build($entity, $tree_to_pass, $langcode);
  }

  /**
   * Render the content of a tree as a tree using a template.
   */
  protected function renderTreeNode(&$content, $entity, EntityCollectionTreeNode $item, $langcode = NULL) {
    $element = array(
      '#collection_children' => array(),
      '#weight' => $item->position,
      '#theme' => 'entity_collection_style_hierarchy'
    );
    // Render the item only if the user has the view permission.
    if (isset($item->content) && entity_access('view', $item->type, $item->content) !== FALSE) {
      $data = EntityCollectionRow::getRow($entity)->build($entity, $item, $langcode);
      $element['#collection_parent'] = $data;
    }
    foreach ($item->getChildren() as $key => $child) {
      $this->renderTreeNode($element['#collection_children'], $entity, $child, $langcode);
    }
    $content[] = $element;
  }

  /**
   * Adds the required settings
   */
  public function settingsForm(&$form, $settings = array()) {
    $settings = $settings + $this->settings;
    parent::settingsForm($form, $settings);

    $limits = !empty($settings['limitation']) ? $settings['limitation'] : array('length'=>0,'offset'=>0);

    $form['limit'] = array(
      '#id' => 'edit-settings-style-limit',
      '#type' => 'checkbox',
      '#title' => t('Display a specified number of items'),
      '#default_value' => ($limits['offset']+$limits['length'] != 0),
    );

    $form['limitation'] = array(
      '#input' => TRUE,
      '#type' => 'fieldset',
      '#states' => array(
        'invisible' => array(
          ':input[id="edit-settings-style-limit"]' => array('checked' => FALSE),
        ),
      ),
    );

    $form['limitation']['length'] = array(
      '#type' => 'textfield',
      '#title' => t('Items to display'),
      '#description' => t('The number of items to display. Enter 0 for no limit.'),
      '#default_value' => $limits['length'],
    );

    $form['limitation']['offset'] = array(
      '#type' => 'textfield',
      '#title' => t('Offset'),
      '#description' => t('The number of items to skip. For example, if this field is 3, the first 3 items will be skipped and not displayed.'),
      '#default_value' => $limits['offset'],
    );

    $form['enable_breakpoint'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable Breakpoint'),
      '#description' => t('Enable/Disable the break functionality. If disabled this will behave like a Hierarchy style.'),
      '#default_value' => !empty($settings['enable_breakpoint']),
    );

    $form['breakpoint'] = array(
      '#type' => 'textfield',
      '#title' => t('Breakpoint'),
      '#description' => t('Set the default breakpoint after a specified number of items. <em>Defaults to: @num</em>', array('@num'=>$this->break_default)),
      '#size' => 40,
      '#maxlength' => 5,
      '#default_value' => isset($settings['breakpoint']) ? $settings['breakpoint'] : $this->break_default,
    );
  }
  
  public function getBreakpoint(EntityCollection $collection) {
    $breakpoint = EntityCollectionStorage::getBundleStorage($collection->bundle)->getBreakpoint($collection, $collection->contexts);
    return (!$breakpoint OR $breakpoint < 0) ? $this->break_default : $breakpoint;
  }
  
  public function setBreakpoint($value=10) {  
    $this->settings['breakpoint'] = $value;
  }
  
  public function useBreakpoint(){
    return !empty($this->settings['enable_breakpoint']);
  }
  
  public function setBreakpointStatus($enabled = TRUE) {
    $this->settings['enable_breakpoint'] = $enabled;
  }

  public function toggleBreakpointStatus($enabled = TRUE) {
    $this->setBreakpointStatus( !$this->useBreakpoint() );
  }

  public function getLimits() {
    return $this->settings['limit'] ? $this->settings['limitation'] : array();
  }

}
